@@ -1,5 +1,5 @@
 ;;; nnmh.el --- mhspool access for Gnus
-;; Copyright (C) 1995,96,97,98 Free Software Foundation, Inc.
+;; Copyright (C) 1995,96,97,98,99 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; 	Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
@@ -60,7 +60,7 @@
 
 (defvoo nnmh-status-string "")
 (defvoo nnmh-group-alist nil)
-(defvoo nnmh-allow-delete-final nil)
+(defvar nnmh-allow-delete-final nil)
 
 
 
@@ -173,11 +173,14 @@
 	       '<))
 	  (cond
 	   (dir
+	    (setq nnmh-group-alist
+		  (delq (assoc group nnmh-group-alist) nnmh-group-alist))
+	    (push (list group (cons (car dir) (car (last dir))))
+		  nnmh-group-alist)
 	    (nnheader-report 'nnmh "Selected group %s" group)
 	    (nnheader-insert
 	     "211 %d %d %d %s\n" (length dir) (car dir)
-	     (progn (while (cdr dir) (setq dir (cdr dir))) (car dir))
-	     group))
+	     (car (last dir)) group))
 	   (t
 	    (nnheader-report 'nnmh "Empty group %s" group)
 	    (nnheader-insert (format "211 0 1 0 %s\n" group))))))))))
@@ -229,8 +232,8 @@
 				(expand-file-name nnmh-toplev))))
 	       dir)
 	      (nnheader-replace-chars-in-string
-	       (gnus-decode-coding-string (substring dir (match-end 0))
-					  nnmail-pathname-coding-system)
+	       (mm-decode-coding-string (substring dir (match-end 0))
+					nnmail-pathname-coding-system)
 	       ?/ ?.))
 	    (apply 'max files)
 	    (apply 'min files)))))))
@@ -242,13 +245,7 @@
 (deffoo nnmh-request-expire-articles (articles newsgroup
 					       &optional server force)
   (nnmh-possibly-change-directory newsgroup server)
-  (let* ((active-articles
-	  (mapcar
-	   (function
-	    (lambda (name)
-	      (string-to-int name)))
-	   (directory-files nnmh-current-directory nil "^[0-9]+$" t)))
-	 (is-old t)
+  (let* ((is-old t)
 	 article rest mod-time)
     (nnheader-init-server-buffer)
 
@@ -456,7 +453,8 @@
   "Compute the next article number in GROUP."
   (let ((active (cadr (assoc group nnmh-group-alist)))
 	(dir (nnmail-group-pathname group nnmh-directory))
-	(pathname-coding-system 'binary))
+	(pathname-coding-system 'binary)
+	file)
     (unless active
       ;; The group wasn't known to nnmh, so we just create an active
       ;; entry for it.
@@ -474,9 +472,15 @@
 	(when files
 	  (setcdr active (car files)))))
     (setcdr active (1+ (cdr active)))
-    (while (file-exists-p
-	    (concat (nnmail-group-pathname group nnmh-directory)
-		    (int-to-string (cdr active))))
+    (while (or
+	    ;; See whether the file exists...
+	    (file-exists-p
+	     (setq file (concat (nnmail-group-pathname group nnmh-directory)
+				(int-to-string (cdr active)))))
+	    ;; ... or there is a buffer that will make that file exist
+	    ;; in the future.
+	    (get-file-buffer file))
+      ;; Skip past that file.
       (setcdr active (1+ (cdr active))))
     (cdr active)))
 
@@ -539,7 +543,7 @@
     (setq articles (sort articles (lambda (art1 art2)
 				    (> (car art1) (car art2)))))
     ;; Finally write this list back to the .nnmh-articles file.
-    (nnheader-temp-write nnmh-file
+    (with-temp-file nnmh-file
       (insert ";; Gnus article active file for " group "\n\n")
       (insert "(setq nnmh-newsgroup-articles '")
       (gnus-prin1 articles)
